#    Society Self-Service
#    Copyright © 2018-2019  Yingtong Li (RunasSudo)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.

import boto3
from botocore.exceptions import ClientError
import premailer

from django.core.management.base import BaseCommand, CommandError

from django.conf import settings
from django.template import loader
from django.urls import reverse
from django.utils import timezone

import ssmembership.models
import sspromotions.models
import sspromotions.utils

import logging
import time
import urllib.parse

def send_aws_email(client, email, subject, content_html, content_txt):
	def send_mail(**kwargs):
		for i in range(0, 10):
			try:
				client.send_email(**kwargs)
				return
			except ClientError as e:
				if e.response['Error']['Code'] == 'Throttling' and e.response['Error']['Message'] == 'Maximum sending rate exceeded.':
					wait_time = max(10*(2**i), 5000)
					self.stdout.write(self.style.NOTICE('Reached maximum sending rate, waiting {} ms'.format(wait_time)))
					time.sleep(wait_time/1000)
				else:
					raise e
		raise Exception('Reached maximum number of retries')
	
	send_mail(
		Destination={
			'ToAddresses': [email],
		},
		Message={
			'Body': {
				'Html': {
					'Charset': 'utf-8',
					'Data': content_html,
				},
				'Text': {
					'Charset': 'utf-8',
					'Data': content_txt,
				},
			},
			'Subject': {
				'Charset': 'utf-8',
				'Data': subject,
			},
		},
		Source='{} <{}>'.format(settings.ORG_NAME, settings.AWS_SENDER_EMAIL),
	)

class Command(BaseCommand):
	help = 'Send bulletin emails'
	
	def add_arguments(self, parser):
		parser.add_argument('ids', nargs='*', type=int, help='Members with ID numbers equal to these values will be emailed (default all)')
		parser.add_argument('--render', action='store_true', help='Render to stdout instead of sending emails')
	
	def handle(self, *args, **options):
		template_html = loader.get_template('sspromotions/email/bulletin.html')
		template_txt = loader.get_template('sspromotions/email/bulletin.txt')
		
		members = ssmembership.models.Member.objects.all()
		
		if len(options['ids']) > 0:
			members = [member for member in members if member.id in options['ids']]
		else:
			raise Exception('Must provide IDs')
		
		client = boto3.client('ses', aws_access_key_id=settings.AWS_KEY_ID, aws_secret_access_key=settings.AWS_SECRET, region_name=settings.AWS_REGION)
		
		title = '{} News: {}'.format(settings.ORG_NAME, timezone.now().strftime('%d %B %Y'))
		
		calbegin, calend, bulbegin, bulend = sspromotions.utils.bulletin_dates(timezone.now())
		events = list(sspromotions.utils.get_calendar_events(calbegin, calend))
		
		for member in members:
			if sspromotions.models.BulletinSubscription.is_member_subscribed(member):
				template_args = sspromotions.utils.bulletin_args(member, sspromotions.models.Group.get_member_groups(member), events, bulbegin, bulend)
				
				content_html = premailer.Premailer(template_html.render(template_args), cssutils_logging_level=logging.ERROR, strip_important=False).transform()
				content_txt = template_txt.render(template_args)
				
				if options['render']:
					self.stdout.write('Content-Type: multipart/alternative; boundary=boundary\n\n--boundary\nContent-Type: text/html; charset=utf-8\n\n' + content_html + '\n--boundary\nContent-Type: text/plain; charset=utf-8\n\n' + content_txt + '\n--boundary')
				else:
					self.stdout.write('Emailing {} at {}'.format(member.id, member.email))
					send_aws_email(client, member.email, title, content_html, content_txt)
